#include <windows.h>
#include <stdio.h>
#include <math.h>

#ifndef ___SHA1_HDR___
#define ___SHA1_HDR___

#if !defined(SHA1_UTILITY_FUNCTIONS) && !defined(SHA1_NO_UTILITY_FUNCTIONS)
#define SHA1_UTILITY_FUNCTIONS
#endif

#include <memory.h> // Needed for memset and memcpy

#ifdef SHA1_UTILITY_FUNCTIONS
#include <stdio.h>  // Needed for file access and sprintf
#include <string.h> // Needed for strcat and strcpy
#endif

#ifdef _MSC_VER
#include <stdlib.h>
#endif

// You can define the endian mode in your files, without modifying the SHA1
// source files. Just #define SHA1_LITTLE_ENDIAN or #define SHA1_BIG_ENDIAN
// in your files, before including the SHA1.h header file. If you don't
// define anything, the class defaults to little endian.

#if !defined(SHA1_LITTLE_ENDIAN) && !defined(SHA1_BIG_ENDIAN)
#define SHA1_LITTLE_ENDIAN
#endif

// Same here. If you want variable wiping, #define SHA1_WIPE_VARIABLES, if
// not, #define SHA1_NO_WIPE_VARIABLES. If you don't define anything, it
// defaults to wiping.

#if !defined(SHA1_WIPE_VARIABLES) && !defined(SHA1_NO_WIPE_VARIABLES)
#define SHA1_WIPE_VARIABLES
#endif

/////////////////////////////////////////////////////////////////////////////
// Define 8- and 32-bit variables

#ifndef UINT_32

#ifdef _MSC_VER

#define UINT_8  unsigned __int8
#define UINT_32 unsigned __int32

#else

#define UINT_8 unsigned char

#if (ULONG_MAX == 0xFFFFFFFF)
#define UINT_32 unsigned long
#else
#define UINT_32 unsigned int
#endif

#endif
#endif

/////////////////////////////////////////////////////////////////////////////
// Declare SHA1 workspace

typedef union
{
	UINT_8  c[64];
	UINT_32 l[16];
} SHA1_WORKSPACE_BLOCK;

class CSHA1
{
public:
#ifdef SHA1_UTILITY_FUNCTIONS
	// Two different formats for ReportHash(...)
	enum
	{
		REPORT_HEX = 0,
		REPORT_DIGIT = 1
	};
#endif

	// Constructor and Destructor
	CSHA1();
	~CSHA1();

	UINT_32 m_state[5];
	UINT_32 m_count[2];
	UINT_32 __reserved1[1];
	UINT_8  m_buffer[64];
	UINT_8  m_digest[20];
	UINT_32 __reserved2[3];

	void Reset();

	// Update the hash value
	void Update(UINT_8 *data, UINT_32 len);


	// Finalize hash and report
	void Final();

	// Report functions: as pre-formatted and raw data
#ifdef SHA1_UTILITY_FUNCTIONS
	void ReportHash(char *szReport, unsigned char uReportType = REPORT_HEX);
#endif
	void GetHash(UINT_8 *puDest);

private:
	// Private SHA-1 transformation
	void Transform(UINT_32 *state, UINT_8 *buffer);

	// Member variables
	UINT_8 m_workspace[64];
	SHA1_WORKSPACE_BLOCK *m_block; // SHA1 pointer to the byte array above
};

#endif


#ifdef SHA1_UTILITY_FUNCTIONS
#define SHA1_MAX_FILE_BUFFER 8000
#endif

// Rotate x bits to the left
#ifndef ROL32
#ifdef _MSC_VER
#define ROL32(_val32, _nBits) _rotl(_val32, _nBits)
#else
#define ROL32(_val32, _nBits) (((_val32)<<(_nBits))|((_val32)>>(32-(_nBits))))
#endif
#endif

#ifdef SHA1_LITTLE_ENDIAN
#define SHABLK0(i) (m_block->l[i] = \
	(ROL32(m_block->l[i],24) & 0xFF00FF00) | (ROL32(m_block->l[i],8) & 0x00FF00FF))
#else
#define SHABLK0(i) (m_block->l[i])
#endif

#define SHABLK(i) (m_block->l[i&15] = ROL32(m_block->l[(i+13)&15] ^ m_block->l[(i+8)&15] \
	^ m_block->l[(i+2)&15] ^ m_block->l[i&15],1))

// SHA-1 rounds
#define _R0(v,w,x,y,z,i) { z+=((w&(x^y))^y)+SHABLK0(i)+0x5A827999+ROL32(v,5); w=ROL32(w,30); }
#define _R1(v,w,x,y,z,i) { z+=((w&(x^y))^y)+SHABLK(i)+0x5A827999+ROL32(v,5); w=ROL32(w,30); }
#define _R2(v,w,x,y,z,i) { z+=(w^x^y)+SHABLK(i)+0x6ED9EBA1+ROL32(v,5); w=ROL32(w,30); }
#define _R3(v,w,x,y,z,i) { z+=(((w|x)&y)|(w&x))+SHABLK(i)+0x8F1BBCDC+ROL32(v,5); w=ROL32(w,30); }
#define _R4(v,w,x,y,z,i) { z+=(w^x^y)+SHABLK(i)+0xCA62C1D6+ROL32(v,5); w=ROL32(w,30); }

CSHA1::CSHA1()
{
	m_block = (SHA1_WORKSPACE_BLOCK *)m_workspace;

	Reset();
}

CSHA1::~CSHA1()
{
	Reset();
}

void CSHA1::Reset()
{
	// SHA1 initialization constants
	m_state[0] = 0x67452301;
	m_state[1] = 0xEFCDAB89;
	m_state[2] = 0x98BADCFE;
	m_state[3] = 0x10325476;
	m_state[4] = 0xC3D2E1F0;

	m_count[0] = 0;
	m_count[1] = 0;
}

void CSHA1::Transform(UINT_32 *state, UINT_8 *buffer)
{
	// Copy state[] to working vars
	UINT_32 a = state[0], b = state[1], c = state[2], d = state[3], e = state[4];

	memcpy(m_block, buffer, 64);

	// 4 rounds of 20 operations each. Loop unrolled.
	_R0(a,b,c,d,e, 0); _R0(e,a,b,c,d, 1); _R0(d,e,a,b,c, 2); _R0(c,d,e,a,b, 3);
	_R0(b,c,d,e,a, 4); _R0(a,b,c,d,e, 5); _R0(e,a,b,c,d, 6); _R0(d,e,a,b,c, 7);
	_R0(c,d,e,a,b, 8); _R0(b,c,d,e,a, 9); _R0(a,b,c,d,e,10); _R0(e,a,b,c,d,11);
	_R0(d,e,a,b,c,12); _R0(c,d,e,a,b,13); _R0(b,c,d,e,a,14); _R0(a,b,c,d,e,15);
	_R1(e,a,b,c,d,16); _R1(d,e,a,b,c,17); _R1(c,d,e,a,b,18); _R1(b,c,d,e,a,19);
	_R2(a,b,c,d,e,20); _R2(e,a,b,c,d,21); _R2(d,e,a,b,c,22); _R2(c,d,e,a,b,23);
	_R2(b,c,d,e,a,24); _R2(a,b,c,d,e,25); _R2(e,a,b,c,d,26); _R2(d,e,a,b,c,27);
	_R2(c,d,e,a,b,28); _R2(b,c,d,e,a,29); _R2(a,b,c,d,e,30); _R2(e,a,b,c,d,31);
	_R2(d,e,a,b,c,32); _R2(c,d,e,a,b,33); _R2(b,c,d,e,a,34); _R2(a,b,c,d,e,35);
	_R2(e,a,b,c,d,36); _R2(d,e,a,b,c,37); _R2(c,d,e,a,b,38); _R2(b,c,d,e,a,39);
	_R3(a,b,c,d,e,40); _R3(e,a,b,c,d,41); _R3(d,e,a,b,c,42); _R3(c,d,e,a,b,43);
	_R3(b,c,d,e,a,44); _R3(a,b,c,d,e,45); _R3(e,a,b,c,d,46); _R3(d,e,a,b,c,47);
	_R3(c,d,e,a,b,48); _R3(b,c,d,e,a,49); _R3(a,b,c,d,e,50); _R3(e,a,b,c,d,51);
	_R3(d,e,a,b,c,52); _R3(c,d,e,a,b,53); _R3(b,c,d,e,a,54); _R3(a,b,c,d,e,55);
	_R3(e,a,b,c,d,56); _R3(d,e,a,b,c,57); _R3(c,d,e,a,b,58); _R3(b,c,d,e,a,59);
	_R4(a,b,c,d,e,60); _R4(e,a,b,c,d,61); _R4(d,e,a,b,c,62); _R4(c,d,e,a,b,63);
	_R4(b,c,d,e,a,64); _R4(a,b,c,d,e,65); _R4(e,a,b,c,d,66); _R4(d,e,a,b,c,67);
	_R4(c,d,e,a,b,68); _R4(b,c,d,e,a,69); _R4(a,b,c,d,e,70); _R4(e,a,b,c,d,71);
	_R4(d,e,a,b,c,72); _R4(c,d,e,a,b,73); _R4(b,c,d,e,a,74); _R4(a,b,c,d,e,75);
	_R4(e,a,b,c,d,76); _R4(d,e,a,b,c,77); _R4(c,d,e,a,b,78); _R4(b,c,d,e,a,79);

	// Add the working vars back into state
	state[0] += a;
	state[1] += b;
	state[2] += c;
	state[3] += d;
	state[4] += e;

	// Wipe variables
#ifdef SHA1_WIPE_VARIABLES
	a = b = c = d = e = 0;
#endif
}

// Use this function to hash in binary data and strings
void CSHA1::Update(UINT_8 *data, UINT_32 len)
{
	UINT_32 i, j;

	j = (m_count[0] >> 3) & 63;

	if((m_count[0] += len << 3) < (len << 3)) m_count[1]++;

	m_count[1] += (len >> 29);

	if((j + len) > 63)
	{
		i = 64 - j;
		memcpy(&m_buffer[j], data, i);
		Transform(m_state, m_buffer);

		for( ; i + 63 < len; i += 64) Transform(m_state, &data[i]);

		j = 0;
	}
	else i = 0;

	memcpy(&m_buffer[j], &data[i], len - i);
}


void CSHA1::Final()
{
	UINT_32 i;
	UINT_8 finalcount[8];

	for(i = 0; i < 8; i++)
		finalcount[i] = (UINT_8)((m_count[((i >= 4) ? 0 : 1)]
			>> ((3 - (i & 3)) * 8) ) & 255); // Endian independent

	Update((UINT_8 *)"\200", 1);

	while ((m_count[0] & 504) != 448)
		Update((UINT_8 *)"\0", 1);

	Update(finalcount, 8); // Cause a SHA1Transform()

	for(i = 0; i < 20; i++)
	{
		m_digest[i] = (UINT_8)((m_state[i >> 2] >> ((3 - (i & 3)) * 8) ) & 255);
	}

	// Wipe variables for security reasons
#ifdef SHA1_WIPE_VARIABLES
	i = 0;
	memset(m_buffer, 0, 64);
	memset(m_state, 0, 20);
	memset(m_count, 0, 8);
	memset(finalcount, 0, 8);
	Transform(m_state, m_buffer);
#endif
}

#ifdef SHA1_UTILITY_FUNCTIONS
// Get the final hash as a pre-formatted string
void CSHA1::ReportHash(char *szReport, unsigned char uReportType)
{
	unsigned char i;
	char szTemp[16];

	if(szReport == NULL) return;

	if(uReportType == REPORT_HEX)
	{
		sprintf(szTemp, "%02X", m_digest[0]);
		strcat(szReport, szTemp);

		for(i = 1; i < 20; i++)
		{
			sprintf(szTemp, "%02X", m_digest[i]);
			strcat(szReport, szTemp);
		}
	}
	else if(uReportType == REPORT_DIGIT)
	{
		sprintf(szTemp, "%u", m_digest[0]);
		strcat(szReport, szTemp);

		for(i = 1; i < 20; i++)
		{
			sprintf(szTemp, "%u", m_digest[i]);
			strcat(szReport, szTemp);
		}
	}
	else strcpy(szReport, "Error: Unknown report type!");
}
#endif

// Get the raw message digest
void CSHA1::GetHash(UINT_8 *puDest)
{
	memcpy(puDest, m_digest, 20);
}

float degtorad(DWORD degrees)
{
        return (degrees % 360) * (float)(3.1415926535f / 180);
}

void encrypt(LPCSTR sourcename, LPCSTR destname, LPCSTR password)
{
char hash1[1024];
memset(hash1,0,1024);
CSHA1 sha1;
sha1.Update((unsigned char *)password, strlen(password));
sha1.Final();
sha1.ReportHash(hash1, CSHA1::REPORT_HEX);
//MessageBoxA(GetActiveWindow(),hash1,"debug",MB_ICONINFORMATION);
CSHA1 sha2;

char revp[1024];
memset(revp,0,1024);
strcpy(revp,password);
strrev(revp);
//MessageBoxA(GetActiveWindow(),revp,"debug",MB_ICONINFORMATION);
char hash2[1024];
memset(hash2,0,1024);

sha2.Update((unsigned char *)revp, strlen(revp));
sha2.Final();
sha2.ReportHash(hash2, CSHA1::REPORT_HEX);
//MessageBoxA(GetActiveWindow(),hash2,"debug",MB_ICONINFORMATION);
//return;
	DeleteFile(destname);
	
	FILE* sfile = fopen(sourcename, "rb");
	if (sfile == NULL)
	{
        MessageBoxA(GetActiveWindow(),sourcename,"Error Opening",MB_ICONINFORMATION);
        return;
	}
	fseek(sfile,-12,SEEK_END);
	char passwordtag[] =  "XMODSIMPLEPW\0\0";
	fgets(passwordtag,13,sfile);
	//MessageBoxA(GetActiveWindow(),passwordtag,"debug",MB_ICONINFORMATION);
	if(strcmp(passwordtag,"FMODSIMPLEPW") == 0)
	{
        fclose(sfile);
        MessageBoxA(GetActiveWindow(),sourcename,"File is protected already!",MB_ICONINFORMATION);
        return;
	}
	fseek(sfile,0,SEEK_SET);
	FILE* dfile = fopen(destname, "wb");
	if (dfile == NULL)
	{
		fclose(sfile);
        MessageBoxA(GetActiveWindow(),destname,"Error Opening",MB_ICONINFORMATION);
        return;
	}
	char c;
	int ln = strlen(hash2);
	int at = 0;
	DWORD rol = 0;
	int lnr = strlen(hash1);
	while (!(feof(sfile)))
	{
        c = fgetc(sfile );
        //fputc((c xor hash2[at]) xor hash1[(int)(fabs(sin(degtorad(rol)))*lnr)],dfile);
        fputc((c xor hash2[at]) xor hash1[at],dfile);
        //fputc(c xor 'A',dfile);
        //fputc(c,dfile);
        //fputc(c xor hash2[at],dfile);
        at++;
        if(at >= ln) at = 0;
        c = 0;
        rol++;
	}
	fputs(hash1, dfile);
	fputs("FMODSIMPLEPW", dfile);
	fclose(sfile);
	fclose(dfile);
}

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)

{
	char files[40000];
	memset(files,0,40000);
	OPENFILENAME ofn2;
    OPENFILENAME ofn;
	char password[2048];
	strcpy(password,"password");
	bool done = false;
	while(done == false)
	{
		done = true;
        memset(&ofn,0,sizeof (OPENFILENAME));
        strcat(files,"select files");
        ofn.lStructSize = sizeof(OPENFILENAME);
        ofn.lpstrFilter = "All Files\0*.*\0\0";
        ofn.lpstrFile = files;
        ofn.nMaxFile = 39999;
        ofn.lpstrTitle = "Select sounds to protect";
        ofn.Flags = OFN_ALLOWMULTISELECT | OFN_EXPLORER | OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_LONGNAMES;
		if(GetOpenFileName(&ofn))
		{
        	memset(&ofn2,0,sizeof (OPENFILENAME));
        	ofn2.lStructSize = sizeof(OPENFILENAME);
        	ofn2.lpstrFilter = "nofiles\0*.\0\0";
        	ofn2.lpstrFile = password;
        	ofn2.nMaxFile = 2047;
        	ofn2.lpstrTitle = "Enter password (sorry for the impromptu input box)";
        	ofn2.Flags = OFN_NOVALIDATE | OFN_NOCHANGEDIR;
            if(GetOpenFileName(&ofn2))
            {
				char *f = files + ofn.nFileOffset;
				char fullsname[2048];
				char fulldname[2048];
				char dfolder[2048];
				memset(fullsname,0,2048);
				memset(fulldname,0,2048);
				memset(dfolder,0,2048);
				strncpy (dfolder,files,ofn.nFileOffset);
				if(dfolder[strlen(dfolder)-1] != '\\') strcat (dfolder,"\\");
				strcat (dfolder,"protected");
				//MessageBoxA(GetActiveWindow(),dfolder,"Folder",MB_ICONINFORMATION);
				CreateDirectory(dfolder,NULL);
				while (f[0] != '\0')
				{
					strncpy (fullsname,files,ofn.nFileOffset);
					if(fullsname[strlen(fullsname)-1] != '\\') strcat (fullsname,"\\");
					strcat (fullsname,f);
					strcpy (fulldname,dfolder);
					strcat (fulldname,"\\");
					strcat (fulldname,f);
					//MessageBoxA(GetActiveWindow(),fullsname,"Source",MB_ICONINFORMATION);
					//MessageBoxA(GetActiveWindow(),fulldname,"Dest",MB_ICONINFORMATION);
					strcpy(password,password+ofn2.nFileOffset);
					//MessageBoxA(GetActiveWindow(),password,"Dest",MB_ICONINFORMATION);
					encrypt(fullsname,fulldname,password);
					f += strlen(f) + 1;
				}
				done =	(MessageBoxA(GetActiveWindow(),"Files are now protected with a password.\n\nYou must now use the password option to play them.\n\nThe new files can be found under the 'protected' folder in the same location.\n\nDo not delete your originals as you may need them if you want to change the password.\n\n\nDo you want to protect more sound files?","Done",MB_ICONINFORMATION|MB_YESNO)==IDNO);
				strncpy (files,files,ofn.nFileOffset);
				files[ofn.nFileOffset] = '\0';
				if(files[strlen(files)-1] != '\\') strcat (files,"\\");
			}
		}

	}
return 1;
}
